@@ -16,7 +16,7 @@ use Log::Log4perl::Appender;
 
 use constant _INTERNAL_DEBUG => 1;
 
-our $VERSION = '1.02';
+our $VERSION = '1.13';
 
    # set this to '1' if you're using a wrapper
    # around Log::Log4perl
@@ -63,6 +63,7 @@ our $CHATTY_DESTROY_METHODS = 0;
 
 our $LOGDIE_MESSAGE_ON_STDERR = 1;
 our $LOGEXIT_CODE             = 1;
+our %IMPORT_CALLED;
 
 ##################################################
 sub import {
@@ -73,6 +74,8 @@ sub import {
 
     my $caller_pkg = caller();
 
+    return 1 if $IMPORT_CALLED{$caller_pkg}++;
+
     my(%tags) = map { $_ => 1 } @_;
 
         # Lazy man's logger
@@ -120,8 +123,9 @@ sub import {
         ${$caller_pkg . '::_default_logger'} = $logger;
         
             # Define DEBUG, INFO, etc. routines in caller's package
-        for(qw(DEBUG INFO WARN ERROR FATAL)) {
+        for(qw(TRACE DEBUG INFO WARN ERROR FATAL ALWAYS)) {
             my $level   = $_;
+            $level = "OFF" if $level eq "ALWAYS";
             my $lclevel = lc($_);
             *{"$caller_pkg\::$_"} = sub { 
                 Log::Log4perl::Logger::init_warn() unless 
@@ -309,7 +313,7 @@ sub easy_init { # Initialize the root logger with a screen appender
             }
             $logger->{file} =~ /^(>)?(>)?/;
             my $mode = ($2 ? "append" : "write");
-            $logger->{file} =~ s/.*>+//g;
+            $logger->{file} =~ s/.*>+\s*//g;
             $app = Log::Log4perl::Appender->new(
                 "Log::Log4perl::Appender::File",
                 filename => $logger->{file},
@@ -641,22 +645,23 @@ In this case, C<Log::Log4perl> will walk up the class hierarchy
 defined somewhere. In the case above, the log level at the root
 (root I<always> defines a log level, but not necessarily an appender)
 defines that 
-the log level is supposed to be C<ERROR> -- meaning that I<debug>
-and I<info> messages are suppressed.
+the log level is supposed to be C<ERROR> -- meaning that I<DEBUG>
+and I<INFO> messages are suppressed.
 
 =head2 Log Levels
 
-There are five predefined log levels: C<FATAL>, C<ERROR>, C<WARN>, C<INFO> 
-and C<DEBUG> (in descending priority). Your configured logging level
+There are five predefined log levels: C<FATAL>, C<ERROR>, C<WARN>, C<INFO>,
+C<DEBUG>, and C<TRACE> (in descending priority). Your configured logging level
 has to at least match the priority of the logging message.
 
 If your configured logging level is C<WARN>, then messages logged 
-with C<info()> and C<debug()> message will be suppressed. 
+with C<info()>, C<debug()>, and C<trace()> will be suppressed. 
 C<fatal()>, C<error()> and C<warn()> will make their way through,
 because their priority is higher or equal than the configured setting.
 
 Instead of calling the methods
 
+    $logger->trace("...");  # Log a trace message
     $logger->debug("...");  # Log a debug message
     $logger->info("...");   # Log a info message
     $logger->warn("...");   # Log a warn message
@@ -668,6 +673,7 @@ using the constants defined in C<Log::Log4perl::Level>:
 
     use Log::Log4perl::Level;
 
+    $logger->log($TRACE, "...");
     $logger->log($DEBUG, "...");
     $logger->log($INFO, "...");
     $logger->log($WARN, "...");
@@ -683,6 +689,7 @@ If you need to find out if the currently configured logging
 level would allow a logger's logging statement to go through, use the
 logger's C<is_I<level>()> methods:
 
+    $logger->is_trace()    # True if trace messages would go through
     $logger->is_debug()    # True if debug messages would go through
     $logger->is_info()     # True if info messages would go through
     $logger->is_warn()     # True if warn messages would go through
@@ -898,7 +905,7 @@ how it works:
     log4j.appender.A1.layout=org.apache.log4j.PatternLayout
     log4j.appender.A1.layout.ConversionPattern=%-4r %-5p %c %x - %m%n
 
-This enables messages of priority C<debug> or higher in the root
+This enables messages of priority C<DEBUG> or higher in the root
 hierarchy and has the system write them to the console. 
 C<ConsoleAppender> is a Java appender, but C<Log::Log4perl> jumps
 through a significant number of hoops internally to map these to their
@@ -1131,8 +1138,7 @@ are located in:
     # ...
 
     package main;
-    Log::Log4perl->init("mylogdefs.conf") or 
-        die "Whoa, cannot read mylogdefs.conf!";
+    Log::Log4perl->init("mylogdefs.conf");
 
         # => "LOG> Creating a new Snickers bar"
     my $first = Candy::Snickers->new();
@@ -1202,15 +1208,15 @@ Here's how it works:
     # Turn off logging in a lower-level category while keeping
     # it active in higher-level categories.
     ############################################################
-    log4perl.rootLogger=debug, LOGFILE
-    log4perl.logger.deep.down.the.hierarchy = error, LOGFILE
+    log4perl.rootLogger=DEBUG, LOGFILE
+    log4perl.logger.deep.down.the.hierarchy = ERROR, LOGFILE
 
     # ... Define appenders ...
 
 This way, log messages issued from within 
 C<Deep::Down::The::Hierarchy> and below will be
-logged only if they're C<error> or worse, while in all other system components
-even C<debug> messages will be logged.
+logged only if they're C<ERROR> or worse, while in all other system components
+even C<DEBUG> messages will be logged.
 
 =head2 Return Values
 
@@ -1418,7 +1424,7 @@ a hash, you can just as well initialize C<Log::Log4perl> with
 a reference to it:
 
     my %key_value_pairs = (
-        "log4perl.rootLogger"       => "error, LOGFILE",
+        "log4perl.rootLogger"       => "ERROR, LOGFILE",
         "log4perl.appender.LOGFILE" => "Log::Log4perl::Appender::File",
         ...
     );
@@ -1702,7 +1708,7 @@ names.
 For more information on opcodes and Safe Compartments, see L<Opcode> and
 L<Safe>.
 
-=head2 Incrementing and Decrementing the Log Levels
+=head2 Changing the Log Level on a Logger
 
 Log4perl provides some internal functions for quickly adjusting the
 log level from within a running Perl program. 
@@ -1715,11 +1721,24 @@ Typically run-time adjusting of levels is done
 at the beginning, or in response to some external input (like a
 "more logging" runtime command for diagnostics).
 
-To increase the level of logging currently being done, use:
+You get the log level from a logger object with:
+
+    $current_level = $logger->level();
+
+and you may set it with the same method, provided you first
+imported the log level constants, with:
+
+    use Log::Log4perl::Level;
+
+Then you can set the level on a logger to one of the constants,
+
+    $logger->level($ERROR); # one of DEBUG, INFO, WARN, ERROR, FATAL
+
+To B<increase> the level of logging currently being done, use:
 
     $logger->more_logging($delta);
 
-and to decrease it, use:
+and to B<decrease> it, use:
 
     $logger->less_logging($delta);
 
@@ -1847,10 +1866,15 @@ your log statements to a file, you can use the following features:
 In C<:easy> mode, C<Log::Log4perl> will instantiate a I<stealth logger>
 named C<$_default_logger> and import it into the current package. Also,
 it will introduce the
-convenience functions C<DEBUG()>, C<INFO()>, C<WARN()>, 
-C<ERROR()> and C<FATAL()> into the package namespace,
-which take arguments and forward them to C<_default_logger-E<gt>debug()>,
+convenience functions C<TRACE>, C<DEBUG()>, C<INFO()>, C<WARN()>, 
+C<ERROR()>, C<FATAL()>, and C<ALWAYS> into the package namespace.
+These functions simply take messages as
+arguments and forward them to C<_default_logger-E<gt>debug()>,
 C<_default_logger-E<gt>info()> and so on.
+If a message should never be blocked, regardless of the log level,
+use the C<ALWAYS> function which corresponds to a log level of C<OFF>:
+
+    ALWAYS "This will be printed regardless of the log level";
 
 The C<easy_init> method can be called with a single level value to
 create a STDERR appender and a root logger as in
@@ -2018,12 +2042,13 @@ the C<get> method:
     my $value = Log::Log4perl::MDC->get($key);
 
 If no value has been stored previously under C<$key>, the C<get> method
-will return the string C<"[undef]"> to allow for easy string interpolation
-later on.
+will return C<undef>.
 
 Typically, MDC values are retrieved later on via the C<"%X{...}"> placeholder
-in C<Log::Log4perl::Layout::PatternLayout>.
-For example, an application taking a web request might store the remote host
+in C<Log::Log4perl::Layout::PatternLayout>. If the C<get()> method
+returns C<undef>, the placeholder will expand to the string C<[undef]>.
+
+An application taking a web request might store the remote host
 like
 
     Log::Log4perl::MDC->put("remote_host", $r->headers("HOST"));
@@ -2099,6 +2124,22 @@ the following:
     ###l4p Log::Log4perl->easy_init($DEBUG);
     ###l4p DEBUG "It works!";
 
+As of C<Log::Log4perl> 1.12, you can even force I<all> modules
+loaded by a script to have their hidden Log4perl statements
+resurrected. For this to happen, load C<Log::Log4perl::Resurrector>
+I<before> loading any modules:
+
+    use Log::Log4perl qw(:easy);
+    use Log::Log4perl::Resurrector;
+
+    use Foobar; # All hidden Log4perl statements in here will
+                # be uncommented before Foobar gets loaded.
+
+    Log::Log4perl->easy_init($DEBUG);
+    ...
+
+Check the C<Log::Log4perl::Resurrector> manpage for more details.
+
 =head2 Access defined appenders
 
 All appenders defined in the configuration file or via Perl code
@@ -2106,6 +2147,22 @@ can be retrieved by the C<appender_by_name()> class method. This comes
 in handy if you want to manipulate or query appender properties after
 the Log4perl configuration has been loaded via C<init()>.
 
+Note that internally, Log::Log4perl uses the C<Log::Log4perl::Appender> 
+wrapper class to control the real appenders (like 
+C<Log::Log4perl::Appender::File> or C<Log::Dispatch::FileRotate>). 
+The C<Log::Log4perl::Appender> class has an C<appender> attribute,
+pointing to the real appender.
+
+The reason for this is that external appenders like 
+C<Log::Dispatch::FileRotate> don't support all of Log::Log4perl's 
+appender control mechanisms (like appender thresholds).
+
+The previously mentioned method C<appender_by_name()> returns a
+referrence to the I<real> appender object. If you want access to the
+wrapper class (e.g. if you want to modify the appender's threshold),
+use the hash C<$Log::Log4perl::Logger::APPENDER_BY_NAME{...}> instead,
+which holds references all appender wrapper objects.
+
 =head2 Modify appender thresholds
 
 To conveniently adjust appender thresholds (e.g. because a script
@@ -2466,25 +2523,16 @@ our
     Mike Schilli <m@perlmeister.com>
     Kevin Goess <cpan@goess.org>
 
-    Contributors:
-    Hutton Davidson <Davidson.Hutton@ftid.com>
-    Chris R. Donnelly <cdonnelly@digitalmotorworks.com>
-    James FitzGibbon <james.fitzgibbon@target.com>
-    Dennis Gregorovic <dgregor@redhat.com>
-    Paul Harrington <Paul-Harrington@deshaw.com>
-    David Hull <hull@paracel.com>
-    Jeff Macdonald <jeff.macdonald@e-dialog.com>
-    Markus Peter <warp@spin.de>
-    Brett Rann <brettrann@mail.com>
-    Erik Selberg <erik@selberg.com>
-    Aaron Straup Cope <asc@vineyard.net>
-    Lars Thegler <lars@thegler.dk>
-    David Viner <dviner@yahoo-inc.com>
-    Mac Yang <mac@proofpoint.com>
+    Contributors (in alphabetical order):
+    Ateeq Altaf, Cory Bennett, Jeremy Bopp, Hutton Davidson, Chris R.
+    Donnelly, Matisse Enzer, Hugh Esco, James FitzGibbon, Carl Franks,
+    Dennis Gregorovic, Paul Harrington, David Hull, Robert Jacobson,
+    Jeff Macdonald, Markus Peter, Brett Rann, Erik Selberg, Aaron
+    Straup Cope, Lars Thegler, David Viner, Mac Yang.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002-2004 by Mike Schilli E<lt>m@perlmeister.comE<gt> and Kevin Goess
+Copyright 2002-2007 by Mike Schilli E<lt>m@perlmeister.comE<gt> and Kevin Goess
 E<lt>cpan@goess.orgE<gt>.
 
 This library is free software; you can redistribute it and/or modify
